@@ -58,6 +58,9 @@
 
 #include <gst/gst.h>
 
+#include <gst/base/gstbasesrc.h>
+
+
 #include "fs-msn-connection.h"
 
 
@@ -100,7 +103,7 @@ struct _FsMsnStreamPrivate
   gint fd;
   gint tos;
 
-  GMutex *mutex; /* protects the conference */
+  GMutex mutex; /* protects the conference */
 };
 
 
@@ -214,7 +217,7 @@ fs_msn_stream_init (FsMsnStream *self)
 
   self->priv->direction = FS_DIRECTION_NONE;
 
-  self->priv->mutex = g_mutex_new ();
+  g_mutex_init (&self->priv->mutex);
 }
 
 
@@ -223,11 +226,11 @@ fs_msn_stream_get_conference (FsMsnStream *self, GError **error)
 {
   FsMsnConference *conference;
 
-  g_mutex_lock (self->priv->mutex);
+  g_mutex_lock (&self->priv->mutex);
   conference = self->priv->conference;
   if (conference)
     g_object_ref (conference);
-  g_mutex_unlock (self->priv->mutex);
+  g_mutex_unlock (&self->priv->mutex);
 
   if (!conference)
     g_set_error (error, FS_ERROR, FS_ERROR_DISPOSED,
@@ -245,9 +248,9 @@ fs_msn_stream_dispose (GObject *object)
   if (!conference)
     return;
 
-  g_mutex_lock (self->priv->mutex);
+  g_mutex_lock (&self->priv->mutex);
   self->priv->conference = NULL;
-  g_mutex_unlock (self->priv->mutex);
+  g_mutex_unlock (&self->priv->mutex);
 
   if (self->priv->src_pad)
   {
@@ -299,7 +302,7 @@ fs_msn_stream_finalize (GObject *object)
 {
   FsMsnStream *self = FS_MSN_STREAM (object);
 
-  g_mutex_free (self->priv->mutex);
+  g_mutex_clear (&self->priv->mutex);
 
   G_OBJECT_CLASS (fs_msn_stream_parent_class)->finalize (object);
 }
@@ -531,7 +534,7 @@ _connected (
         "fdsrc name=fdsrc do-timestamp=true ! mimdec ! valve name=recv_valve", TRUE, &error);
   else
     codecbin = gst_parse_bin_from_description (
-        "ffmpegcolorspace ! videoscale ! mimenc name=enc !"
+        "videoconvert ! videoscale ! mimenc name=enc !"
         " fdsink name=fdsink sync=false async=false",
         TRUE, &error);
 
@@ -555,9 +558,14 @@ _connected (
   }
 
   if (self->priv->conference->max_direction == FS_DIRECTION_RECV)
+  {
     fdelem = gst_bin_get_by_name (GST_BIN (codecbin), "fdsrc");
+    gst_base_src_set_format (GST_BASE_SRC (fdelem), GST_FORMAT_TIME);
+  }
   else
+  {
     fdelem = gst_bin_get_by_name (GST_BIN (codecbin), "fdsink");
+  }
 
   if (!fdelem)
   {
