@@ -643,11 +643,6 @@ fs_rawudp_component_stop (FsRawUdpComponent *self)
         fs_rawudp_transmitter_udpport_remove_dest (udpport,
             self->priv->remote_candidate->ip,
             self->priv->remote_candidate->port);
-      else
-        fs_rawudp_transmitter_udpport_remove_recvonly_dest (udpport,
-            self->priv->remote_candidate->ip,
-            self->priv->remote_candidate->port);
-
 
       fs_rawudp_transmitter_udpport_remove_known_address (udpport,
           self->priv->remote_address, remote_is_unique_cb, self);
@@ -761,8 +756,6 @@ fs_rawudp_component_set_property (GObject *object,
         {
           if (sending)
           {
-            fs_rawudp_transmitter_udpport_remove_recvonly_dest (
-                self->priv->udpport, candidate->ip, candidate->port);
             fs_rawudp_transmitter_udpport_add_dest (self->priv->udpport,
                 candidate->ip, candidate->port);
           }
@@ -770,8 +763,6 @@ fs_rawudp_component_set_property (GObject *object,
           {
             fs_rawudp_transmitter_udpport_remove_dest (self->priv->udpport,
                 candidate->ip, candidate->port);
-            fs_rawudp_transmitter_udpport_add_recvonly_dest (
-                self->priv->udpport, candidate->ip, candidate->port);
           }
         }
         if (candidate)
@@ -972,9 +963,6 @@ fs_rawudp_component_set_remote_candidate (FsRawUdpComponent *self,
   if (sending)
     fs_rawudp_transmitter_udpport_add_dest (self->priv->udpport,
         candidate->ip, candidate->port);
-  else
-    fs_rawudp_transmitter_udpport_add_recvonly_dest (self->priv->udpport,
-        candidate->ip, candidate->port);
 
   if (old_candidate)
   {
@@ -982,10 +970,6 @@ fs_rawudp_component_set_remote_candidate (FsRawUdpComponent *self,
       fs_rawudp_transmitter_udpport_remove_dest (self->priv->udpport,
           old_candidate->ip,
           old_candidate->port);
-    else
-      fs_rawudp_transmitter_udpport_remove_recvonly_dest (self->priv->udpport,
-          candidate->ip,
-          candidate->port);
     fs_candidate_destroy (old_candidate);
   }
 
